package hw.tagApi.service.service.impl;

import java.io.ByteArrayInputStream;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

import hw.tagApi.common.exception.ServiceException;
import hw.tagApi.common.utils.DateUtils;
import hw.tagApi.common.utils.uuid.IdGenerator;
import hw.tagApi.service.constant.ApiConstants;
import hw.tagApi.service.domain.ApiContent;
import hw.tagApi.service.utils.TagExcelUtil;
import hw.tagApi.service.utils.httpClientUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import hw.tagApi.service.mapper.HwTagRecordMapper;
import hw.tagApi.service.domain.HwTagRecord;
import hw.tagApi.service.service.IHwTagRecordService;
import org.springframework.transaction.annotation.Transactional;

import static hw.tagApi.service.utils.TagExcelUtil.startsWithCheck;

/**
 * 标签记录Service业务层处理
 *
 * @author Yinq
 * @date 2025-04-15
 */
@Service
public class HwTagRecordServiceImpl implements IHwTagRecordService {

    private static final Logger log = LoggerFactory.getLogger(HwTagRecordServiceImpl.class);

    @Autowired
    private HwTagRecordMapper hwTagRecordMapper;

    private static final int BATCH_SIZE = 100;

    /**
     * 查询标签记录
     *
     * @param rId 标签记录主键
     * @return 标签记录
     */
    @Override
    public HwTagRecord selectHwTagRecordByRId(Long rId) {
        return hwTagRecordMapper.selectHwTagRecordByRId(rId);
    }

    /**
     * 查询标签记录列表
     *
     * @param hwTagRecord 标签记录
     * @return 标签记录
     */
    @Override
    public List<HwTagRecord> selectHwTagRecordList(HwTagRecord hwTagRecord) {
        return hwTagRecordMapper.selectHwTagRecordList(hwTagRecord);
    }

    /**
     * 新增标签记录
     *
     * @param hwTagRecord 标签记录
     * @return 结果
     */
    @Override
    public int insertHwTagRecord(HwTagRecord hwTagRecord) {
        hwTagRecord.setCreateTime(DateUtils.getNowDate());
        return hwTagRecordMapper.insertHwTagRecord(hwTagRecord);
    }

    /**
     * 修改标签记录
     *
     * @param hwTagRecord 标签记录
     * @return 结果
     */
    @Override
    public int updateHwTagRecord(HwTagRecord hwTagRecord) {
        initTagRecord(hwTagRecord);
        return hwTagRecordMapper.updateHwTagRecord(hwTagRecord);
    }

    /**
     * 初始化hwTagRecord
     *
     * @param hwTagRecord hwTagRecord
     */
    public void initTagRecord(HwTagRecord hwTagRecord) {
        hwTagRecord.setRId(IdGenerator.nextId());
        hwTagRecord.setCreateTime(DateUtils.getNowDate());
    }

    /**
     * 批量删除标签记录
     *
     * @param rIds 需要删除的标签记录主键
     * @return 结果
     */
    @Override
    public int deleteHwTagRecordByRIds(Long[] rIds) {
        return hwTagRecordMapper.deleteHwTagRecordByRIds(rIds);
    }

    /**
     * 删除标签记录信息
     *
     * @param rId 标签记录主键
     * @return 结果
     */
    @Override
    public int deleteHwTagRecordByRId(Long rId) {
        return hwTagRecordMapper.deleteHwTagRecordByRId(rId);
    }

    /**
     * 根据TID更新标签记录
     *
     * @param record 标签记录
     */
    @Override
    public void updateHwTagRecordByTID(HwTagRecord record) {


        hwTagRecordMapper.updateHwTagRecordByTID(record);
    }

    /**
     * 批量插入标签记录
     *
     * @param batchList 标签记录列表
     */
    @Override
    public void batchInsertHwTagRecord(List<HwTagRecord> batchList) {
        hwTagRecordMapper.batchInsertHwTagRecord(batchList);
    }

    /**
     * 查询接口根据条件查询标签记录
     *
     * @param selectTagRecord （HwTagRecord）
     * @return HwTagRecord
     */
    @Override
    public HwTagRecord selectHwTagRecordData(HwTagRecord selectTagRecord) {
        List<HwTagRecord> tagRecordList = hwTagRecordMapper.selectHwTagRecordList(selectTagRecord);
        if (!tagRecordList.isEmpty()) {
            HwTagRecord tagRecord = tagRecordList.get(0);
            tagRecord.setQueryResultsNumber(tagRecordList.size());
            return tagRecord;
        }
        return null;
    }

    /**
     * 根据TID查询标签记录
     *
     * @param tId TID
     */
    @Override
    public HwTagRecord selectHwTagRecordByTID(String tId) {
        return hwTagRecordMapper.selectHwTagRecordByTxId(tId);
    }

    /**
     * 批量查询已存在的TID
     *
     * @param tids TID列表
     * @return 已存在的TID集合
     */
    @Override
    public Set<String> selectExistingTids(List<String> tids) {
        if (tids == null || tids.isEmpty()) {
            return new HashSet<>();
        }
        return new HashSet<>(hwTagRecordMapper.selectExistingTids(tids));
    }

    /**
     * 批量查询已存在的EPC
     *
     * @param epcs EPC列表
     * @return 已存在的EPC集合
     */
    @Override
    public Set<String> selectExistingEpcs(List<String> epcs) {
        if (epcs == null || epcs.isEmpty()) {
            return new HashSet<>();
        }
        return new HashSet<>(hwTagRecordMapper.selectExistingEpcs(epcs));
    }

    /**
     * 导入数据
     *
     * @param apiContent 包含导入数据的ApiContent对象
     * @return 处理后的ApiContent对象
     */
    @Transactional(rollbackFor = Exception.class)
    public ApiContent importData(ApiContent apiContent) {
        Map<String, Object> fields = apiContent.getFields();
        String importMode = String.valueOf(fields.get("导入模式"));
        String model = String.valueOf(fields.getOrDefault("型号", ""));
        String fileUrl = String.valueOf(fields.get("文件链接"));
        String fileName = String.valueOf(fields.getOrDefault("文件名", ""));
        String TIDCheck = String.valueOf(fields.getOrDefault("TID校验", ""));
        String EPCCheck = String.valueOf(fields.getOrDefault("EPC校验", ""));
        String destinationInfo = String.valueOf(fields.getOrDefault("去向信息", ""));
        String libraryFlag = String.valueOf(fields.getOrDefault("是否在库", ""));
        String intendedUse = String.valueOf(fields.getOrDefault("计划用途", ""));
        String remark = String.valueOf(fields.getOrDefault("备注", ""));
        String EPCNumberCheck = String.valueOf(fields.getOrDefault("EPC位数校验", "0"));
        int EPCNumberCheckLength = Integer.parseInt(EPCNumberCheck);
        int tidCharCount = TIDCheck.length();
        int epcCharCount = EPCCheck.length();
        List<HwTagRecord> batchList = new ArrayList<>();
        Map<String, Object> resFields = new HashMap<>();
        List<String> errorMessages = new ArrayList<>();
        Date nowDate = new Date();
        int successCount = 0;
        int failCount = 0;
        apiContent.setFields(resFields);
        try {
            log.info("开始处理文件，URL: {}", fileUrl);
            // 获取文件字节数组
            byte[] fileData = httpClientUtils.getByteArrayFromUrl(fileUrl);
            if (fileData == null) {
                log.error("获取文件失败");
                resFields.put(ApiConstants.IMPORT_STATUS, "系统错误");
                resFields.put(ApiConstants.DESCRIPTION, "获取文件失败");
                return apiContent;
            }

            // 解析Excel数据
            try (ByteArrayInputStream bis = new ByteArrayInputStream(fileData)) {
                Map<String, Object> excelResult = TagExcelUtil.parseTagExcel(bis);
                if (!excelResult.containsKey("tagList")) {
                    log.error("解析Excel文件失败，未找到标签数据");
                    resFields.put(ApiConstants.IMPORT_STATUS, "导入失败");
                    resFields.put(ApiConstants.DESCRIPTION, "解析Excel文件失败，未找到标签数据");
                    return apiContent;
                }
                // 获取订单信息
                Map<String, String> orderInfo = (Map<String, String>) excelResult.get("orderInfo");
                // 获取标签数据列表
                List<Map<String, String>> tagList = (List<Map<String, String>>) excelResult.get("tagList");
                // 批量处理标签数据
                SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

                // 如果不是更新模式，先批量查询所有TID
                Set<String> existingTids = new HashSet<>();
                boolean tIdFlag = ApiConstants.MODE_ADD.equals(importMode) || ApiConstants.MODE_ADD_NO_CHECK.equals(importMode);
                if (tIdFlag) {
                    List<String> allTids = tagList.stream()
                            .map(tag -> tag.get("TID").replace("-", ""))
                            .collect(Collectors.toList());
                    existingTids = this.selectExistingTids(allTids);
                }
                // 如果不是更新模式，批量查询所有EPC
                Set<String> existingEpcs = new HashSet<>();
                boolean epcCheckFlag = ApiConstants.MODE_ADD.equals(importMode) || ApiConstants.MODE_UPDATE.equals(importMode);
                if (epcCheckFlag) {
                    List<String> allEpcs = tagList.stream()
                            .map(tag -> tag.get("EPC").replace("-", ""))
                            .collect(Collectors.toList());
                    existingEpcs = this.selectExistingEpcs(allEpcs);
                }

                for (Map<String, String> tagData : tagList) {
                    try {
                        String tid = tagData.get("TID").replace("-", "");
                        String epc = tagData.get("EPC").replace("-", "");
                        if (tid.length() != ApiConstants.TE_LENGTH || (EPCNumberCheckLength > 0 && epc.length() != EPCNumberCheckLength)) {
                            StringBuilder builder = new StringBuilder();
                            if (tid.length() != ApiConstants.TE_LENGTH) {
                                String errorMsg = String.format("TID长度校验失败！预期长度：24，实际长度：%d，TID值：%s", tid.length(), tid);
                                builder.append(errorMsg);
                                log.error("异常 - {}", errorMsg);
                            }
                            if (EPCNumberCheckLength > 0 && epc.length() != EPCNumberCheckLength) {
                                String errorMsg = String.format("EPC长度校验失败！预期长度：24，实际长度：%d，EPC值：%s", epc.length(), epc);
                                builder.append(errorMsg);
                                log.error("EPC长度校验异常 - {}", errorMsg);
                            }
                            errorMessages.add(builder.toString());
                            failCount++;
                            continue;
                        }
                        if (tidCharCount > 0 && !startsWithCheck(tid, TIDCheck)) {
                            String errorMsg = String.format("TID校验失败！TID校验：%s，实际TID：%s", TIDCheck, tid);
                            log.error("TID校验异常 - {}", errorMsg);
                            errorMessages.add(errorMsg);
                            failCount++;
                            continue;
                        }
                        if (epcCharCount > 0 && !startsWithCheck(epc, EPCCheck)) {
                            String errorMsg = String.format("EPC校验失败！EPC校验：%s，实际EPC：%s", EPCCheck, epc);
                            log.error("EPC校验异常 - {}", errorMsg);
                            errorMessages.add(errorMsg);
                            failCount++;
                            continue;
                        }
                        HwTagRecord record = new HwTagRecord();
                        String lot = orderInfo.getOrDefault("orderNo","").replace("Lot", "");
                        record.setOrderCode(lot);
                        record.setBatchNumber(orderInfo.getOrDefault("batchNo", ""));
                        record.setOperatorId(orderInfo.getOrDefault("operatorId", ""));
                        Date processTime = orderInfo.containsKey("processTime") ? sdf.parse(orderInfo.get("processTime")) : nowDate;
                        record.setProcessingTime(processTime);
                        record.setTotalQuantity(Long.valueOf(orderInfo.getOrDefault("totalCount", "0")));
                        // 设置标签数据
                        record.setModelCode(model);
                        record.setTagSequence(tagData.get("序号"));
                        record.setTId(tid);
                        record.setEpc(epc);
                        record.setPassword(tagData.get("密码"));
                        record.setTestResult(tagData.get("测试结果"));
                        record.setTestValue(tagData.get("测试值"));
                        record.setReferenceValue(tagData.get("参考值"));
                        Date testingTime = tagData.containsKey("测试时间") ? sdf.parse(tagData.get("测试时间")) : nowDate;
                        record.setTestingTime(testingTime);
                        record.setFileName(fileName);
                        record.setDestinationInfo(destinationInfo);
                        record.setLibraryFlag(libraryFlag);
                        record.setIntendedUse(intendedUse);
                        record.setRemark(remark);

                        //校验tId唯一
                        if (tIdFlag && existingTids.contains(tid)) {
                            String errorMsg = String.format("TID已存在：%s", tid);
                            log.error(errorMsg);
                            errorMessages.add(errorMsg);
                            failCount++;
                            continue;
                        }
                        //校验EPC唯一
                        if (epcCheckFlag && existingEpcs.contains(epc)) {
                            String errorMsg = String.format("EPC已存在：%s", epc);
                            log.error(errorMsg);
                            errorMessages.add(errorMsg);
                            failCount++;
                            continue;
                        }
                        // 更新方法
                        if ((ApiConstants.MODE_UPDATE.equals(importMode) || ApiConstants.MODE_UPDATE_NO_CHECK.equals(importMode)) && failCount == 0) {
                            this.updateHwTagRecordByTID(record);
                            successCount++;
                            continue;
                        }
                        record.setRId(IdGenerator.nextId());
                        // 添加到批处理列表
                        batchList.add(record);
                        // 达到批量大小时执行插入
                        if (batchList.size() >= BATCH_SIZE && failCount == 0) {
                            this.batchInsertHwTagRecord(batchList);
                            log.info("批量插入{}条数据成功", batchList.size());
                            successCount += batchList.size();
                            batchList.clear();
                        }
                    } catch (Exception e) {
                        String errorMsg = String.format("处理数据失败：%s", e.getMessage());
                        log.error(errorMsg);
                        errorMessages.add(errorMsg);
                        failCount++;
                    }
                }

                // 处理剩余的数据
                if (!batchList.isEmpty() && failCount == 0) {
                    this.batchInsertHwTagRecord(batchList);
                    log.info("批量插入剩余{}条数据成功", batchList.size());
                    successCount += batchList.size();
                    batchList.clear();
                }

                // 设置导入结果
                if (failCount > 0) {
                    throw new ServiceException(String.format(importMode + "失败：%d条。失败原因：%s",
                            failCount, String.join("; ", errorMessages)));
                } else {
                    resFields.put(ApiConstants.IMPORT_STATUS, "导入成功");
                    resFields.put(ApiConstants.DESCRIPTION, String.format("成功导入%d条数据", successCount));
                }
            }
        } catch (Exception e) {
            log.error("处理文件失败: {}", e.getMessage(), e);
            resFields.put(ApiConstants.IMPORT_STATUS, "导入失败");
            resFields.put(ApiConstants.DESCRIPTION, e.getMessage());
        }

        return apiContent;
    }


}
