package hw.tagApi.service.service.impl;

import hw.tagApi.service.service.IHwTagRecordService;
import hw.tagApi.service.service.IKDocsService;
import hw.tagApi.service.utils.TagExcelUtil;
import hw.tagApi.service.utils.httpClientUtils;

import java.time.Duration;

import org.springframework.stereotype.Service;
import org.springframework.beans.factory.annotation.Autowired;

import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.*;

import hw.tagApi.service.constant.ApiConstants;
import hw.tagApi.service.domain.HwTagRecord;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import hw.tagApi.service.domain.ApiContent;
import org.springframework.util.StringUtils;


/**
 * 云文档服务实现类
 *
 * @author Yinq
 * @date 2025-04-15
 */
@Service
public class KDocsServiceImpl implements IKDocsService {
    private static final Logger log = LoggerFactory.getLogger(KDocsServiceImpl.class);

    @Autowired
    private IHwTagRecordService tagRecordService;

    // 创建线程池
    private static final ExecutorService executorService = new ThreadPoolExecutor(
            5,                      // 核心线程数
            10,                    // 最大线程数
            60L,                   // 空闲线程存活时间
            TimeUnit.SECONDS,      // 时间单位
            new LinkedBlockingQueue<>(100),  // 任务队列
            new ThreadPoolExecutor.CallerRunsPolicy()  // 拒绝策略：由调用线程处理
    );

    /**
     * 导入数据列表
     *
     * @param contentList 包含导入数据的ApiContent对象列表
     * @return 初始响应的ApiContent对象列表
     */
    @Override
    public List<ApiContent> importDataList(List<ApiContent> contentList) {
        List<ApiContent> resApiContentList = getApiContents(contentList);

        // 异步执行导入任务
        executorService.execute(() -> {
            List<ApiContent> resList = new ArrayList<>();
            for (ApiContent apiContent : contentList) {
                long startTime = System.currentTimeMillis();
                ApiContent resContent = new ApiContent();
                resContent.setId(apiContent.getId());
                try {
                    ApiContent result = tagRecordService.importData(apiContent);
                    // 更新响应内容
                    resContent.setFields(result.getFields());
                } catch (Exception e) {
                    log.error("导入任务执行失败: {}", e.getMessage(), e);
                    Map<String, Object> errorFields = new HashMap<>();
                    errorFields.put(ApiConstants.IMPORT_STATUS, "导入失败");
                    errorFields.put(ApiConstants.DESCRIPTION, "导入任务执行失败: " + e.getMessage());
                    resContent.setFields(errorFields);
                }
                resList.add(resContent);
                long endTime = System.currentTimeMillis();
                log.info("数据库导入完成 - ID: {}, 耗时: {}ms", apiContent.getId(), (endTime - startTime));
            }

            // 发送处理结果到回调接口
            httpClientUtils.sendResultToCallback(resList, ApiConstants.IN_CALLBACK_URL, ApiConstants.DATA_IMPORT);
        });

        return resApiContentList;
    }

    /**
     * 创建初始响应的ApiContent对象列表
     *
     * @param contentList 包含导入数据的ApiContent对象列表
     * @return 初始响应的ApiContent对象列表
     */
    private static List<ApiContent> getApiContents(List<ApiContent> contentList) {
        List<ApiContent> resApiContentList = new ArrayList<>();
        for (ApiContent apiContent : contentList) {
            // 创建初始响应
            Map<String, Object> resFields = new HashMap<>();
            resFields.put(ApiConstants.IMPORT_STATUS, "导入中");
            resFields.put(ApiConstants.DESCRIPTION, "后台处理中");
            ApiContent responseContent = new ApiContent();
            responseContent.setId(apiContent.getId());
            responseContent.setFields(resFields);
            resApiContentList.add(responseContent);
        }
        return resApiContentList;
    }

    /**
     * 查询数据
     *
     * @param content 查询条件列表
     * @return 查询结果列表
     */
    @Override
    public List<ApiContent> queryDataList(List<ApiContent> content) {
        List<ApiContent> result = new ArrayList<>();
        if (content == null || content.isEmpty()) {
            log.warn("查询条件列表为空");
            return result;
        }

        log.info("开始处理查询请求，查询条件数量: {}", content.size());
        for (ApiContent query : content) {
            String queryId = query.getId();
            log.info("处理查询条件 ID: {}, 查询参数: {}", queryId, query.getFields());

            ApiContent response = new ApiContent();
            response.setId(queryId);
            response.setFields(new HashMap<>());
            Map<String, Object> fields = query.getFields();
            HwTagRecord selectTagRecord = new HwTagRecord();

            // 检查查询条件
            if (!fields.containsKey("TID") && !fields.containsKey("EPC")) {
                log.warn("查询条件错误 - ID: {}, 缺少必要的查询参数(TID或EPC)", queryId);
                response.getFields().put(ApiConstants.SELECT_STATUS, "系统错误");
                response.getFields().put(ApiConstants.DESCRIPTION, "查询条件必须包含TID或EPC");
                result.add(response);
                continue;
            }

            // 设置查询条件
            if (fields.containsKey("TID")) {
                String tid = (String) fields.get("TID");
                selectTagRecord.setTId(tid);
            }
            if (fields.containsKey("EPC")) {
                String epc = (String) fields.get("EPC");
                selectTagRecord.setEpc(epc);
            }
            if (fields.containsKey("搜索日期起点")) {
                String startDate = (String) fields.get("搜索日期起点");
                selectTagRecord.setStartDate(startDate);
            }
            if (fields.containsKey("搜索日期终点")) {
                String endDate = (String) fields.get("搜索日期终点");
                selectTagRecord.setEndDate(endDate);
            }

            // 执行查询
            log.info("执行数据库查询 - ID: {}", queryId);
            long startTime = System.currentTimeMillis();
            HwTagRecord data = tagRecordService.selectHwTagRecordData(selectTagRecord);
            long endTime = System.currentTimeMillis();
            log.info("数据库查询完成 - ID: {}, 耗时: {}ms", queryId, (endTime - startTime));

            // 处理查询结果
            if (data != null) {
                log.info("查询成功 - ID: {}, TID: {}, EPC: {}", queryId, data.getTId(), data.getEpc());
                // 查询成功
                if (data.getQueryResultsNumber() > 1){
                    response.getFields().put(ApiConstants.SELECT_STATUS, "系统错误");
                    response.getFields().put(ApiConstants.DESCRIPTION, "存在多个记录，请使用数据导出功能");
                } else {
                    response.getFields().put(ApiConstants.SELECT_STATUS, "搜索成功");
                }
                response.getFields().put("TID", data.getTId());
                response.getFields().put("EPC", data.getEpc());
                response.getFields().put("标签序号", data.getTagSequence());
                response.getFields().put("密码", data.getPassword() != null ? data.getPassword() : "");
                response.getFields().put("RID", data.getTId());
                response.getFields().put("LOT", data.getOrderCode());
                response.getFields().put("卷编号", data.getBatchNumber());
                response.getFields().put("本卷数量", data.getTotalQuantity() != null ? data.getTotalQuantity() : "");
                response.getFields().put("标签批次号", data.getTagBatch() != null ? data.getTagBatch() : "");
                response.getFields().put("型号", data.getModelCode() != null ? data.getModelCode() : "");
                response.getFields().put("加工时间", new SimpleDateFormat("yyyy/MM/dd HH:mm:ss").format(data.getProcessingTime()));
                response.getFields().put("生产人员编号", data.getOperatorId());
                response.getFields().put("去向信息", data.getDestinationInfo() != null ? data.getDestinationInfo() : "");
                response.getFields().put("是否在库", data.getLibraryFlag() != null ? data.getLibraryFlag() : "");
                response.getFields().put("计划用途", data.getIntendedUse() != null ? data.getIntendedUse() : "");
                response.getFields().put("备注", data.getRemark() != null ? data.getRemark() : "");
                log.debug("查询结果详情 - ID: {}, 结果: {}", queryId, response.getFields());
            } else {
                log.info("未找到记录 - ID: {}", queryId);
                // 未找到记录
                response.getFields().put(ApiConstants.SELECT_STATUS, "未找到记录");
                if (fields.containsKey("搜索日期起点") && fields.containsKey("搜索日期终点")) {
                    String message = String.format("已遍历整个数据库，生产日期从%s到%s，未找到匹配的记录。",
                            fields.get("搜索日期起点"), fields.get("搜索日期终点"));
                    response.getFields().put(ApiConstants.DESCRIPTION, message);
                    log.debug("未找到记录详情 - ID: {}, 日期范围: {} - {}",
                            queryId, fields.get("搜索日期起点"), fields.get("搜索日期终点"));
                } else {
                    response.getFields().put(ApiConstants.DESCRIPTION, "未找到匹配的记录");
                }
            }

            result.add(response);
        }

        return result;
    }

    /**
     * 导出数据
     *
     * @param content 导出条件列表
     * @return 导出结果列表
     */
    @Override
    public List<ApiContent> exportDataList(List<ApiContent> content) {
        List<ApiContent> result = new ArrayList<>();
        if (content == null || content.isEmpty()) {
            log.warn("导出条件列表为空");
            return result;
        }
        log.info("开始处理导出请求，导出条件数量: {}", content.size());
        // 创建初始响应
        for (ApiContent condition : content) {
            ApiContent response = new ApiContent();
            response.setId(condition.getId());
            Map<String, Object> responseFields = new HashMap<>();
            responseFields.put(ApiConstants.EXPORT_STATUS, "导出中");
            responseFields.put(ApiConstants.DESCRIPTION, "预计需要3分钟");
            response.setFields(responseFields);
            result.add(response);
        }

        // 异步执行导出任务
        executorService.execute(() -> {
            List<ApiContent> resApiContentList = new ArrayList<>();
            for (ApiContent condition : content) {
                ApiContent response = new ApiContent();
                response.setId(condition.getId());
                try {
                    Map<String, Object> fields = condition.getFields();
                    String lot = fields.get("LOT") != null ? (String) fields.get("LOT") : "";
                    String startDate = fields.get("导出日期起点") != null ? (String) fields.get("导出日期起点") : "";
                    String endDate = fields.get("导出日期终点") != null ? (String) fields.get("导出日期终点") : "";
                    String tagBatch = fields.get("标签批次号") != null ? (String) fields.get("标签批次号") : "";
                    String batchNumber = fields.get("卷编号") != null ? (String) fields.get("卷编号") : "";
                    String modelCode = fields.get("型号") != null ? (String) fields.get("型号") : "";
                    String TID = fields.get("TID") != null ? (String) fields.get("TID") : "";
                    String EPC = fields.get("EPC") != null ? (String) fields.get("EPC") : "";

                    // 构建查询条件
                    HwTagRecord queryCondition = new HwTagRecord();
                    if (StringUtils.hasText(lot)) {
                        queryCondition.setOrderCode(lot);
                    }
                    if (StringUtils.hasText(startDate)) {
                        queryCondition.setStartDate(startDate);
                    }
                    if (StringUtils.hasText(endDate)) {
                        queryCondition.setEndDate(endDate);
                    }
                    if (StringUtils.hasText(tagBatch)) {
                        queryCondition.setTagBatch(tagBatch);
                    }
                    if (StringUtils.hasText(batchNumber)) {
                        queryCondition.setBatchNumber(batchNumber);
                    }
                    if (StringUtils.hasText(modelCode)) {
                        queryCondition.setModelCode(modelCode);
                    }
                    if (StringUtils.hasText(TID)) {
                        queryCondition.setSelectTID(TID);
                    }
                    if (StringUtils.hasText(EPC)) {
                        queryCondition.setSelectEPC(EPC);
                    }
                    // 执行查询
                    log.info("执行数据导出查询 - ID: {}", condition.getId());
                    long startTime = System.currentTimeMillis();
                    List<HwTagRecord> records = tagRecordService.selectHwTagRecordList(queryCondition);


                    Map<String, Object> responseFields = new HashMap<>();

                    if (records != null && !records.isEmpty()) {
                        try {
                            String fileName = String.format("标签数据_%s.xlsx", condition.getId());
                            String filePath = TagExcelUtil.generateTagExcel(records, fileName);
                            long endTime = System.currentTimeMillis();
                            Duration duration = Duration.ofMillis(endTime - startTime);
                            long minutes = duration.toMinutes();
                            long seconds = duration.minusMinutes(minutes).getSeconds();
                            Date futureDate = new Date(endTime + ApiConstants.PASS_TIME);
                            SimpleDateFormat sdf = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss");
                            log.info("数据库导出完成 - ID: {}, 记录数: {}, 耗时: {}分{}秒", condition.getId(), records.size(), minutes, seconds);
                            responseFields.put(ApiConstants.EXPORT_STATUS, "导出成功");
                            responseFields.put(ApiConstants.DESCRIPTION, "耗时: " + minutes + "分" + seconds + "秒");
                            responseFields.put("文件链接", filePath);
                            responseFields.put("链接有效期", sdf.format(futureDate));
                            log.info("导出文件生成成功 - ID: {}, 路径: {}", condition.getId(), filePath);
                        } catch (Exception e) {
                            log.error("生成导出文件失败 - ID: {}", condition.getId(), e);
                            responseFields.put(ApiConstants.EXPORT_STATUS, "导出失败");
                            responseFields.put(ApiConstants.DESCRIPTION, "生成导出文件失败: " + e.getMessage());
                        }
                    } else {
                        responseFields.put(ApiConstants.EXPORT_STATUS, "未找到记录");
                        responseFields.put(ApiConstants.DESCRIPTION, "未找到符合条件的记录");
                    }

                    response.setFields(responseFields);

                } catch (Exception e) {
                    log.error("处理导出任务失败 - ID: {}", condition.getId(), e);
                    response.setId(condition.getId());
                    Map<String, Object> responseFields = new HashMap<>();
                    responseFields.put(ApiConstants.EXPORT_STATUS, "导出失败");
                    responseFields.put(ApiConstants.DESCRIPTION, "处理导出任务失败: " + e.getMessage());
                    response.setFields(responseFields);

                }
                resApiContentList.add(response);
            }
            // 发送处理结果到回调接口
            httpClientUtils.sendResultToCallback(resApiContentList, ApiConstants.OUT_CALLBACK_URL, ApiConstants.DATA_EXPORT);
        });

        return result;
    }

    /**
     * 快速导入数据
     *
     * @param content 数据内容
     * @return 处理结果
     */
    @Override
    public List<ApiContent> fastImportDataList(List<ApiContent> content) {
        List<ApiContent> resList = new ArrayList<>();
        for (ApiContent apiContent : content) {
            long startTime = System.currentTimeMillis();
            ApiContent resContent = new ApiContent();
            resContent.setId(apiContent.getId());
            try {
                ApiContent result = tagRecordService.importData(apiContent);
                // 更新响应内容
                resContent.setFields(result.getFields());
            } catch (Exception e) {
                log.error("快速导入任务执行失败: {}", e.getMessage(), e);
                Map<String, Object> errorFields = new HashMap<>();
                errorFields.put(ApiConstants.IMPORT_STATUS, "导入失败");
                errorFields.put(ApiConstants.DESCRIPTION, "快速导入任务执行失败: " + e.getMessage());
                resContent.setFields(errorFields);
            }
            resList.add(resContent);
            long endTime = System.currentTimeMillis();
            log.info("数据库快速导入完成 - ID: {}, 耗时: {}ms", apiContent.getId(), (endTime - startTime));
        }
        return resList;
    }


}
